-- Do not edit; this file was generated by Cheby using these options:
--  --gen-hdl=dma_controller_regs.vhd -i dma_controller_regs.cheby

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity dma_controller_regs is
  port (
    rst_n_i              : in    std_logic;
    clk_i                : in    std_logic;
    wb_cyc_i             : in    std_logic;
    wb_stb_i             : in    std_logic;
    wb_adr_i             : in    std_logic_vector(5 downto 2);
    wb_sel_i             : in    std_logic_vector(3 downto 0);
    wb_we_i              : in    std_logic;
    wb_dat_i             : in    std_logic_vector(31 downto 0);
    wb_ack_o             : out   std_logic;
    wb_err_o             : out   std_logic;
    wb_rty_o             : out   std_logic;
    wb_stall_o           : out   std_logic;
    wb_dat_o             : out   std_logic_vector(31 downto 0);

    -- Start DMA transfer
    ctrl_start_i         : in    std_logic;
    ctrl_start_o         : out   std_logic;

    -- Abort current DMA transfer
    ctrl_abort_i         : in    std_logic;
    ctrl_abort_o         : out   std_logic;

    -- Control byte-swapping
    ctrl_byteswap_o      : out   std_logic_vector(1 downto 0);
    ctrl_wr_o            : out   std_logic;
    ctrl_wack_i          : in    std_logic;

    -- DMA state
    stat_status_i        : in    std_logic_vector(1 downto 0);
    stat_status_o        : out   std_logic_vector(1 downto 0);

    -- IRQ status, write 1 to clear
    stat_irq_i           : in    std_logic;
    stat_irq_o           : out   std_logic;
    stat_wr_o            : out   std_logic;
    stat_rd_o            : out   std_logic;
    stat_wack_i          : in    std_logic;
    stat_rack_i          : in    std_logic;

    -- Carrier start address
    cstart_o             : out   std_logic_vector(31 downto 0);

    -- DMA start address (low) in the host
    hstartl_o            : out   std_logic_vector(31 downto 0);

    -- DMA start address (high) in the host
    hstarth_o            : out   std_logic_vector(31 downto 0);

    -- DMA read length in bytes
    len_o                : out   std_logic_vector(31 downto 0);

    -- Pointer (low) to next item in list
    nextl_o              : out   std_logic_vector(31 downto 0);

    -- Pointer (high) to next item in list
    nexth_o              : out   std_logic_vector(31 downto 0);

    -- Transfer chain (if 1)
    attrib_chain_o       : out   std_logic;

    -- Transfer direction (0: to host, 1: to carrier)
    attrib_dir_o         : out   std_logic
  );
end dma_controller_regs;

architecture syn of dma_controller_regs is
  signal rd_int                         : std_logic;
  signal wr_int                         : std_logic;
  signal rd_ack_int                     : std_logic;
  signal wr_ack_int                     : std_logic;
  signal wb_en                          : std_logic;
  signal ack_int                        : std_logic;
  signal wb_rip                         : std_logic;
  signal wb_wip                         : std_logic;
  signal ctrl_byteswap_reg              : std_logic_vector(1 downto 0);
  signal cstart_reg                     : std_logic_vector(31 downto 0);
  signal hstartl_reg                    : std_logic_vector(31 downto 0);
  signal hstarth_reg                    : std_logic_vector(31 downto 0);
  signal len_reg                        : std_logic_vector(31 downto 0);
  signal nextl_reg                      : std_logic_vector(31 downto 0);
  signal nexth_reg                      : std_logic_vector(31 downto 0);
  signal attrib_chain_reg               : std_logic;
  signal attrib_dir_reg                 : std_logic;
  signal reg_rdat_int                   : std_logic_vector(31 downto 0);
  signal rd_ack1_int                    : std_logic;
begin

  -- WB decode signals
  wb_en <= wb_cyc_i and wb_stb_i;

  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        wb_rip <= '0';
      else
        wb_rip <= (wb_rip or (wb_en and not wb_we_i)) and not rd_ack_int;
      end if;
    end if;
  end process;
  rd_int <= (wb_en and not wb_we_i) and not wb_rip;

  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        wb_wip <= '0';
      else
        wb_wip <= (wb_wip or (wb_en and wb_we_i)) and not wr_ack_int;
      end if;
    end if;
  end process;
  wr_int <= (wb_en and wb_we_i) and not wb_wip;

  ack_int <= rd_ack_int or wr_ack_int;
  wb_ack_o <= ack_int;
  wb_stall_o <= not ack_int and wb_en;
  wb_rty_o <= '0';
  wb_err_o <= '0';

  -- Assign outputs
  ctrl_byteswap_o <= ctrl_byteswap_reg;
  cstart_o <= cstart_reg;
  hstartl_o <= hstartl_reg;
  hstarth_o <= hstarth_reg;
  len_o <= len_reg;
  nextl_o <= nextl_reg;
  nexth_o <= nexth_reg;
  attrib_chain_o <= attrib_chain_reg;
  attrib_dir_o <= attrib_dir_reg;

  -- Process for write requests.
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        wr_ack_int <= '0';
        ctrl_wr_o <= '0';
        ctrl_byteswap_reg <= "00";
        stat_wr_o <= '0';
        cstart_reg <= "00000000000000000000000000000000";
        hstartl_reg <= "00000000000000000000000000000000";
        hstarth_reg <= "00000000000000000000000000000000";
        len_reg <= "00000000000000000000000000000000";
        nextl_reg <= "00000000000000000000000000000000";
        nexth_reg <= "00000000000000000000000000000000";
        attrib_chain_reg <= '0';
        attrib_dir_reg <= '0';
      else
        wr_ack_int <= '0';
        ctrl_wr_o <= '0';
        stat_wr_o <= '0';
        case wb_adr_i(5 downto 2) is
        when "0000" => 
          -- Register ctrl
          ctrl_wr_o <= wr_int;
          if wr_int = '1' then
            ctrl_start_o <= wb_dat_i(0);
            ctrl_abort_o <= wb_dat_i(1);
            ctrl_byteswap_reg <= wb_dat_i(3 downto 2);
          end if;
          wr_ack_int <= ctrl_wack_i;
        when "0001" => 
          -- Register stat
          stat_wr_o <= wr_int;
          if wr_int = '1' then
            stat_status_o <= wb_dat_i(1 downto 0);
            stat_irq_o <= wb_dat_i(2);
          end if;
          wr_ack_int <= stat_wack_i;
        when "0010" => 
          -- Register cstart
          if wr_int = '1' then
            cstart_reg <= wb_dat_i;
          end if;
          wr_ack_int <= wr_int;
        when "0011" => 
          -- Register hstartl
          if wr_int = '1' then
            hstartl_reg <= wb_dat_i;
          end if;
          wr_ack_int <= wr_int;
        when "0100" => 
          -- Register hstarth
          if wr_int = '1' then
            hstarth_reg <= wb_dat_i;
          end if;
          wr_ack_int <= wr_int;
        when "0101" => 
          -- Register len
          if wr_int = '1' then
            len_reg <= wb_dat_i;
          end if;
          wr_ack_int <= wr_int;
        when "0110" => 
          -- Register nextl
          if wr_int = '1' then
            nextl_reg <= wb_dat_i;
          end if;
          wr_ack_int <= wr_int;
        when "0111" => 
          -- Register nexth
          if wr_int = '1' then
            nexth_reg <= wb_dat_i;
          end if;
          wr_ack_int <= wr_int;
        when "1000" => 
          -- Register attrib
          if wr_int = '1' then
            attrib_chain_reg <= wb_dat_i(1);
            attrib_dir_reg <= wb_dat_i(0);
          end if;
          wr_ack_int <= wr_int;
        when others =>
          wr_ack_int <= wr_int;
        end case;
      end if;
    end if;
  end process;

  -- Process for registers read.
  process (clk_i) begin
    if rising_edge(clk_i) then
      if rst_n_i = '0' then
        rd_ack1_int <= '0';
        stat_rd_o <= '0';
      else
        stat_rd_o <= '0';
        reg_rdat_int <= (others => '0');
        case wb_adr_i(5 downto 2) is
        when "0000" => 
          -- ctrl
          reg_rdat_int(0) <= ctrl_start_i;
          reg_rdat_int(1) <= ctrl_abort_i;
          reg_rdat_int(3 downto 2) <= ctrl_byteswap_reg;
          rd_ack1_int <= rd_int;
        when "0001" => 
          -- stat
          reg_rdat_int(1 downto 0) <= stat_status_i;
          reg_rdat_int(2) <= stat_irq_i;
          stat_rd_o <= rd_int;
          rd_ack1_int <= stat_rack_i;
        when "0010" => 
          -- cstart
          reg_rdat_int <= cstart_reg;
          rd_ack1_int <= rd_int;
        when "0011" => 
          -- hstartl
          reg_rdat_int <= hstartl_reg;
          rd_ack1_int <= rd_int;
        when "0100" => 
          -- hstarth
          reg_rdat_int <= hstarth_reg;
          rd_ack1_int <= rd_int;
        when "0101" => 
          -- len
          reg_rdat_int <= len_reg;
          rd_ack1_int <= rd_int;
        when "0110" => 
          -- nextl
          reg_rdat_int <= nextl_reg;
          rd_ack1_int <= rd_int;
        when "0111" => 
          -- nexth
          reg_rdat_int <= nexth_reg;
          rd_ack1_int <= rd_int;
        when "1000" => 
          -- attrib
          reg_rdat_int(1) <= attrib_chain_reg;
          reg_rdat_int(0) <= attrib_dir_reg;
          rd_ack1_int <= rd_int;
        when others =>
          reg_rdat_int <= (others => 'X');
          rd_ack1_int <= rd_int;
        end case;
      end if;
    end if;
  end process;

  -- Process for read requests.
  process (wb_adr_i, reg_rdat_int, rd_ack1_int, rd_int) begin
    -- By default ack read requests
    wb_dat_o <= (others => '0');
    case wb_adr_i(5 downto 2) is
    when "0000" => 
      -- ctrl
      wb_dat_o <= reg_rdat_int;
      rd_ack_int <= rd_ack1_int;
    when "0001" => 
      -- stat
      wb_dat_o <= reg_rdat_int;
      rd_ack_int <= rd_ack1_int;
    when "0010" => 
      -- cstart
      wb_dat_o <= reg_rdat_int;
      rd_ack_int <= rd_ack1_int;
    when "0011" => 
      -- hstartl
      wb_dat_o <= reg_rdat_int;
      rd_ack_int <= rd_ack1_int;
    when "0100" => 
      -- hstarth
      wb_dat_o <= reg_rdat_int;
      rd_ack_int <= rd_ack1_int;
    when "0101" => 
      -- len
      wb_dat_o <= reg_rdat_int;
      rd_ack_int <= rd_ack1_int;
    when "0110" => 
      -- nextl
      wb_dat_o <= reg_rdat_int;
      rd_ack_int <= rd_ack1_int;
    when "0111" => 
      -- nexth
      wb_dat_o <= reg_rdat_int;
      rd_ack_int <= rd_ack1_int;
    when "1000" => 
      -- attrib
      wb_dat_o <= reg_rdat_int;
      rd_ack_int <= rd_ack1_int;
    when others =>
      rd_ack_int <= rd_int;
    end case;
  end process;
end syn;
